@@ -34,3 +34,18 @@ class AWithQuotes:
     correct_code: 'list[int]' = KNOWINGLY_MUTABLE_DEFAULT
     perfectly_fine: 'list[int]' = field(default_factory=list)
     class_variable: 'typing.ClassVar[list[int]]'= []
+
+
+# Mutable defaults wrapped in field() calls
+@dataclass
+class C:
+    mutable_default: list[int] = field(default=[])  # RUF008
+    mutable_default2: dict[str, int] = field(default={})  # RUF008
+    mutable_default3: set[int] = field(default=set())  # RUF008
+    mutable_default4: dict[str, int] = field(default=dict())  # RUF008
+    correct_factory: list[int] = field(default_factory=list)  # okay
+    immutable_default: tuple[int, ...] = field(default=())  # okay
+    immutable_default2: str = field(default="hello")  # okay
+    immutable_default3: int = field(default=1)  # okay
+    non_mutable_var: list[int] = field(default=KNOWINGLY_MUTABLE_DEFAULT)  # okay
+    class_variable: ClassVar[list[int]] = field(default=[])  # okay
