@@ -265,3 +265,25 @@ pub(crate) const fn is_ble001_exc_info_suppression_enabled(settings: &LinterSett
 pub(crate) const fn is_py315_support_enabled(settings: &LinterSettings) -> bool {
     settings.preview.is_enabled()
 }
+
+// https://github.com/astral-sh/ruff/pull/23046
+pub(crate) const fn is_mutable_default_in_dataclass_field_enabled(
+    settings: &LinterSettings,
+) -> bool {
+    settings.preview.is_enabled()
+}
+
+// https://github.com/astral-sh/ruff/pull/22830
+pub(crate) const fn is_standalone_mock_non_existent_enabled(settings: &LinterSettings) -> bool {
+    settings.preview.is_enabled()
+}
+
+// https://github.com/astral-sh/ruff/pull/23013
+pub(crate) const fn is_up024_precise_highlighting_enabled(settings: &LinterSettings) -> bool {
+    settings.preview.is_enabled()
+}
+
+// https://github.com/astral-sh/ruff/pull/21078
+pub(crate) const fn is_plural_ngettext_check_enabled(settings: &LinterSettings) -> bool {
+    settings.preview.is_enabled()
+}
