@@ -1,5 +1,6 @@
 pub(crate) use dag_schedule_argument::*;
 pub(crate) use function_signature_change_in_3::*;
+pub(crate) use moved_in_3_1::*;
 pub(crate) use moved_to_provider_in_3::*;
 pub(crate) use removal_in_3::*;
 pub(crate) use suggested_to_move_to_provider_in_3::*;
@@ -8,6 +9,7 @@ pub(crate) use task_variable_name::*;
 
 mod dag_schedule_argument;
 mod function_signature_change_in_3;
+mod moved_in_3_1;
 mod moved_to_provider_in_3;
 mod removal_in_3;
 mod suggested_to_move_to_provider_in_3;
