@@ -331,7 +331,7 @@ mod tests {
 
     #[test]
     fn range_suppressions() -> Result<()> {
-        assert_diagnostics_diff!(
+        let diagnostics = test_path(
             Path::new("ruff/suppressions.py"),
             &settings::LinterSettings::for_rules(vec![
                 Rule::UnusedVariable,
@@ -342,17 +342,8 @@ mod tests {
                 Rule::UnmatchedSuppressionComment,
             ])
             .with_external_rules(&["TK421"]),
-            &settings::LinterSettings::for_rules(vec![
-                Rule::UnusedVariable,
-                Rule::AmbiguousVariableName,
-                Rule::UnusedNOQA,
-                Rule::InvalidRuleCode,
-                Rule::InvalidSuppressionComment,
-                Rule::UnmatchedSuppressionComment,
-            ])
-            .with_external_rules(&["TK421"])
-            .with_preview_mode(),
-        );
+        )?;
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -611,6 +602,8 @@ mod tests {
         Ok(())
     }
 
+    #[test_case(Rule::MutableDataclassDefault, Path::new("RUF008.py"))]
+    #[test_case(Rule::MutableDataclassDefault, Path::new("RUF008_attrs.py"))]
     #[test_case(Rule::UnrawRePattern, Path::new("RUF039.py"))]
     #[test_case(Rule::UnrawRePattern, Path::new("RUF039_concat.py"))]
     #[test_case(Rule::UnnecessaryRegularExpression, Path::new("RUF055_0.py"))]
@@ -619,6 +612,7 @@ mod tests {
     #[test_case(Rule::UnnecessaryRegularExpression, Path::new("RUF055_3.py"))]
     #[test_case(Rule::IndentedFormFeed, Path::new("RUF054.py"))]
     #[test_case(Rule::ImplicitClassVarInDataclass, Path::new("RUF045.py"))]
+    #[test_case(Rule::FloatEqualityComparison, Path::new("RUF069.py"))]
     fn preview_rules(rule_code: Rule, path: &Path) -> Result<()> {
         let snapshot = format!(
             "preview__{}_{}",
