@@ -382,68 +382,74 @@ impl SalsaMemoryDump {
         DisplayFull(self)
     }
 
-    /// Returns a redacted report that provides rounded totals of memory usage, to avoid
-    /// overly sensitive diffs in `mypy-primer` runs.
-    pub fn display_mypy_primer(&self) -> impl fmt::Display + '_ {
-        struct DisplayShort<'a>(&'a SalsaMemoryDump);
-
-        fn round_memory(total: usize) -> usize {
-            // Round the number to the nearest power of 1.05. This gives us a
-            // 2.5% threshold before the memory usage number is considered to have
-            // changed.
-            //
-            // TODO: Small changes in memory usage may cause the number to be rounded
-            // into the next power if it happened to already be close to the threshold.
-            // This also means that differences may surface as a result of small changes
-            // over time that are unrelated to the current change. Ideally we could compare
-            // the exact numbers across runs and compute the difference, but we don't have
-            // the infrastructure for that currently.
-            const BASE: f64 = 1.05;
-            BASE.powf(bytes_to_mb(total).log(BASE).round()) as usize
+    /// Serializes the memory dump to JSON.
+    pub fn to_json(&self) -> String {
+        #[derive(serde::Serialize)]
+        struct MemoryReport {
+            total_bytes: usize,
+            struct_metadata_bytes: usize,
+            struct_fields_bytes: usize,
+            memo_metadata_bytes: usize,
+            memo_fields_bytes: usize,
+            structs: Vec<IngredientReport>,
+            queries: Vec<QueryReport>,
         }
 
-        impl fmt::Display for DisplayShort<'_> {
-            fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-                let SalsaMemoryDump {
-                    total_fields,
-                    total_metadata,
-                    total_memo_fields,
-                    total_memo_metadata,
-                    ..
-                } = *self.0;
-
-                writeln!(f, "=======SALSA SUMMARY=======")?;
-
-                writeln!(
-                    f,
-                    "TOTAL MEMORY USAGE: ~{}MB",
-                    round_memory(
-                        total_metadata + total_fields + total_memo_fields + total_memo_metadata
-                    )
-                )?;
-
-                writeln!(
-                    f,
-                    "    struct metadata = ~{}MB",
-                    round_memory(total_metadata)
-                )?;
-                writeln!(f, "    struct fields = ~{}MB", round_memory(total_fields))?;
-                writeln!(
-                    f,
-                    "    memo metadata = ~{}MB",
-                    round_memory(total_memo_metadata)
-                )?;
-                writeln!(
-                    f,
-                    "    memo fields = ~{}MB",
-                    round_memory(total_memo_fields)
-                )?;
-
-                Ok(())
-            }
+        #[derive(serde::Serialize)]
+        struct IngredientReport {
+            name: String,
+            metadata_bytes: usize,
+            fields_bytes: usize,
+            count: usize,
         }
 
-        DisplayShort(self)
+        #[derive(serde::Serialize)]
+        struct QueryReport {
+            name: String,
+            return_type: String,
+            metadata_bytes: usize,
+            fields_bytes: usize,
+            count: usize,
+        }
+
+        let structs = self
+            .ingredients
+            .iter()
+            .map(|ingredient| IngredientReport {
+                name: ingredient.debug_name().to_string(),
+                metadata_bytes: ingredient.size_of_metadata(),
+                fields_bytes: ingredient.size_of_fields()
+                    + ingredient.heap_size_of_fields().unwrap_or(0),
+                count: ingredient.count(),
+            })
+            .collect();
+
+        let queries = self
+            .memos
+            .iter()
+            .map(|(query_fn, memo)| QueryReport {
+                name: (*query_fn).to_string(),
+                return_type: memo.debug_name().to_string(),
+                metadata_bytes: memo.size_of_metadata(),
+                fields_bytes: memo.size_of_fields() + memo.heap_size_of_fields().unwrap_or(0),
+                count: memo.count(),
+            })
+            .collect();
+
+        let report = MemoryReport {
+            total_bytes: self.total_fields
+                + self.total_metadata
+                + self.total_memo_fields
+                + self.total_memo_metadata,
+            struct_metadata_bytes: self.total_metadata,
+            struct_fields_bytes: self.total_fields,
+            memo_metadata_bytes: self.total_memo_metadata,
+            memo_fields_bytes: self.total_memo_fields,
+            structs,
+            queries,
+        };
+
+        serde_json::to_string_pretty(&report).expect("Failed to serialize memory report")
     }
 }
 
@@ -536,6 +542,7 @@ pub(crate) mod tests {
     use ruff_db::files::{FileRootKind, Files};
     use ruff_db::system::{DbWithTestSystem, System, TestSystem};
     use ruff_db::vendored::VendoredFileSystem;
+    use ruff_python_ast::PythonVersion;
     use ty_module_resolver::SearchPathSettings;
     use ty_python_semantic::lint::{LintRegistry, RuleSelection};
     use ty_python_semantic::{
@@ -582,6 +589,13 @@ pub(crate) mod tests {
         }
 
         pub fn init_program(&mut self) -> anyhow::Result<()> {
+            self.init_program_with_python_version(PythonVersion::latest_ty())
+        }
+
+        pub fn init_program_with_python_version(
+            &mut self,
+            python_version: PythonVersion,
+        ) -> anyhow::Result<()> {
             let root = self.project().root(self);
 
             let search_paths = SearchPathSettings::new(vec![root.to_path_buf()])
@@ -591,7 +605,10 @@ pub(crate) mod tests {
             Program::from_settings(
                 self,
                 ProgramSettings {
-                    python_version: PythonVersionWithSource::default(),
+                    python_version: PythonVersionWithSource {
+                        source: ty_python_semantic::PythonVersionSource::Default,
+                        version: python_version,
+                    },
                     python_platform: PythonPlatform::default(),
                     search_paths,
                 },
