@@ -4,15 +4,14 @@ use std::sync::Arc;
 use ruff_db::files::File;
 use ruff_db::parsed::parsed_module;
 use ruff_index::{IndexSlice, IndexVec};
-
 use ruff_python_ast::NodeIndex;
 use ruff_python_parser::semantic_errors::SemanticSyntaxError;
 use rustc_hash::{FxHashMap, FxHashSet};
 use salsa::Update;
 use salsa::plumbing::AsId;
+use ty_module_resolver::ModuleName;
 
 use crate::Db;
-use crate::module_name::ModuleName;
 use crate::node_key::NodeKey;
 use crate::semantic_index::ast_ids::AstIds;
 use crate::semantic_index::ast_ids::node_key::ExpressionNodeKey;
@@ -470,6 +469,11 @@ impl<'db> SemanticIndex<'db> {
     ///
     /// If the number of definitions associated with the key is not exactly 1 and
     /// the `debug_assertions` feature is enabled, this method will panic.
+    ///
+    /// It is generally safe to use this method for any AST node that does not
+    /// correspond to a `*` (wildcard) import, since `*` imports are the only
+    /// situations that can result in multiple definitions being associated with a
+    /// single AST node.
     #[track_caller]
     pub(crate) fn expect_single_definition(
         &self,
@@ -527,6 +531,15 @@ impl<'db> SemanticIndex<'db> {
         self.scopes_by_node.get(&node.node_key()).copied()
     }
 
+    /// Returns the id of the scope that the node identified by `key` creates.
+    ///
+    /// This is useful when you have a [`NodeWithScopeKey`] constructed from an
+    /// [`AstNodeRef`](crate::ast_node_ref::AstNodeRef) and want to avoid loading
+    /// the parsed module just to look up the scope.
+    pub(crate) fn node_scope_by_key(&self, key: NodeWithScopeKey) -> FileScopeId {
+        self.scopes_by_node[&key]
+    }
+
     /// Checks if there is an import of `__future__.annotations` in the global scope, which affects
     /// the logic for type inference.
     pub(super) fn has_future_annotations(&self) -> bool {
@@ -782,6 +795,13 @@ mod tests {
                 .find_map(|constrained_binding| constrained_binding.binding.definition())
         }
 
+        fn first_public_declaration(&self, symbol: ScopedSymbolId) -> Option<Definition<'_>> {
+            self.end_of_scope_symbol_declarations(symbol)
+                .find_map(|declaration_with_constraint| {
+                    declaration_with_constraint.declaration.definition()
+                })
+        }
+
         fn first_binding_at_use(&self, use_id: ScopedUseId) -> Option<Definition<'_>> {
             self.bindings_at_use(use_id)
                 .find_map(|constrained_binding| constrained_binding.binding.definition())
@@ -834,10 +854,19 @@ mod tests {
     #[test]
     fn annotation_only() {
         let TestCase { db, file } = test_case("x: int");
-        let global_table = place_table(&db, global_scope(&db, file));
+        let scope = global_scope(&db, file);
+        let global_table = place_table(&db, scope);
 
         assert_eq!(names(global_table), vec!["int", "x"]);
-        // TODO record definition
+
+        let use_def = use_def_map(&db, scope);
+        let declaration = use_def
+            .first_public_declaration(global_table.symbol_id("x").expect("symbol to exist"))
+            .unwrap();
+        assert!(matches!(
+            declaration.kind(&db),
+            DefinitionKind::AnnotatedAssignment(_)
+        ));
     }
 
     #[test]
