@@ -248,13 +248,10 @@ use crate::place::BoundnessAnalysis;
 use crate::semantic_index::ast_ids::ScopedUseId;
 use crate::semantic_index::definition::{Definition, DefinitionState};
 use crate::semantic_index::member::ScopedMemberId;
-use crate::semantic_index::narrowing_constraints::{
-    ConstraintKey, NarrowingConstraints, NarrowingConstraintsBuilder, NarrowingConstraintsIterator,
-    ScopedNarrowingConstraint,
-};
+use crate::semantic_index::narrowing_constraints::{ConstraintKey, ScopedNarrowingConstraint};
 use crate::semantic_index::place::{PlaceExprRef, ScopedPlaceId};
 use crate::semantic_index::predicate::{
-    Predicate, PredicateOrLiteral, Predicates, PredicatesBuilder, ScopedPredicateId,
+    PredicateOrLiteral, Predicates, PredicatesBuilder, ScopedPredicateId,
 };
 use crate::semantic_index::reachability_constraints::{
     ReachabilityConstraints, ReachabilityConstraintsBuilder, ScopedReachabilityConstraintId,
@@ -266,7 +263,7 @@ use crate::semantic_index::use_def::place_state::{
     LiveDeclarationsIterator, PlaceState, PreviousDefinitions, ScopedDefinitionId,
 };
 use crate::semantic_index::{EnclosingSnapshotResult, SemanticIndex};
-use crate::types::{NarrowingConstraint, Truthiness, Type, infer_narrowing_constraint};
+use crate::types::{PossiblyNarrowedPlaces, Truthiness, Type};
 
 mod place_state;
 
@@ -280,9 +277,6 @@ pub(crate) struct UseDefMap<'db> {
     /// Array of predicates in this scope.
     predicates: Predicates<'db>,
 
-    /// Array of narrowing constraints in this scope.
-    narrowing_constraints: NarrowingConstraints,
-
     /// Array of reachability constraints in this scope.
     reachability_constraints: ReachabilityConstraints,
 
@@ -349,7 +343,7 @@ pub(crate) struct UseDefMap<'db> {
 }
 
 pub(crate) enum ApplicableConstraints<'map, 'db> {
-    UnboundBinding(ConstraintsIterator<'map, 'db>),
+    UnboundBinding(NarrowingEvaluator<'map, 'db>),
     ConstrainedBindings(BindingWithConstraintsIterator<'map, 'db>),
 }
 
@@ -373,9 +367,10 @@ impl<'db> UseDefMap<'db> {
     ) -> ApplicableConstraints<'_, 'db> {
         match constraint_key {
             ConstraintKey::NarrowingConstraint(constraint) => {
-                ApplicableConstraints::UnboundBinding(ConstraintsIterator {
+                ApplicableConstraints::UnboundBinding(NarrowingEvaluator {
+                    constraint,
                     predicates: &self.predicates,
-                    constraint_ids: self.narrowing_constraints.iter_predicates(constraint),
+                    reachability_constraints: &self.reachability_constraints,
                 })
             }
             ConstraintKey::NestedScope(nested_scope) => {
@@ -642,7 +637,6 @@ impl<'db> UseDefMap<'db> {
         BindingWithConstraintsIterator {
             all_definitions: &self.all_definitions,
             predicates: &self.predicates,
-            narrowing_constraints: &self.narrowing_constraints,
             reachability_constraints: &self.reachability_constraints,
             boundness_analysis,
             inner: bindings.iter(),
@@ -698,7 +692,6 @@ type EnclosingSnapshots = IndexVec<ScopedEnclosingSnapshotId, EnclosingSnapshot>
 pub(crate) struct BindingWithConstraintsIterator<'map, 'db> {
     all_definitions: &'map IndexVec<ScopedDefinitionId, DefinitionState<'db>>,
     pub(crate) predicates: &'map Predicates<'db>,
-    pub(crate) narrowing_constraints: &'map NarrowingConstraints,
     pub(crate) reachability_constraints: &'map ReachabilityConstraints,
     pub(crate) boundness_analysis: BoundnessAnalysis,
     inner: LiveBindingsIterator<'map>,
@@ -709,16 +702,16 @@ impl<'map, 'db> Iterator for BindingWithConstraintsIterator<'map, 'db> {
 
     fn next(&mut self) -> Option<Self::Item> {
         let predicates = self.predicates;
-        let narrowing_constraints = self.narrowing_constraints;
+        let reachability_constraints = self.reachability_constraints;
 
         self.inner
             .next()
             .map(|live_binding| BindingWithConstraints {
                 binding: self.all_definitions[live_binding.binding],
-                narrowing_constraint: ConstraintsIterator {
+                narrowing_constraint: NarrowingEvaluator {
+                    constraint: live_binding.narrowing_constraint,
                     predicates,
-                    constraint_ids: narrowing_constraints
-                        .iter_predicates(live_binding.narrowing_constraint),
+                    reachability_constraints,
                 },
                 reachability_constraint: live_binding.reachability_constraint,
             })
@@ -729,50 +722,30 @@ impl std::iter::FusedIterator for BindingWithConstraintsIterator<'_, '_> {}
 
 pub(crate) struct BindingWithConstraints<'map, 'db> {
     pub(crate) binding: DefinitionState<'db>,
-    pub(crate) narrowing_constraint: ConstraintsIterator<'map, 'db>,
+    pub(crate) narrowing_constraint: NarrowingEvaluator<'map, 'db>,
     pub(crate) reachability_constraint: ScopedReachabilityConstraintId,
 }
 
-pub(crate) struct ConstraintsIterator<'map, 'db> {
+pub(crate) struct NarrowingEvaluator<'map, 'db> {
+    pub(crate) constraint: ScopedNarrowingConstraint,
     predicates: &'map Predicates<'db>,
-    constraint_ids: NarrowingConstraintsIterator<'map>,
+    reachability_constraints: &'map ReachabilityConstraints,
 }
 
-impl<'db> Iterator for ConstraintsIterator<'_, 'db> {
-    type Item = Predicate<'db>;
-
-    fn next(&mut self) -> Option<Self::Item> {
-        self.constraint_ids
-            .next()
-            .map(|narrowing_constraint| self.predicates[narrowing_constraint.predicate()])
-    }
-}
-
-impl std::iter::FusedIterator for ConstraintsIterator<'_, '_> {}
-
-impl<'db> ConstraintsIterator<'_, 'db> {
+impl<'db> NarrowingEvaluator<'_, 'db> {
     pub(crate) fn narrow(
         self,
         db: &'db dyn crate::Db,
         base_ty: Type<'db>,
         place: ScopedPlaceId,
     ) -> Type<'db> {
-        // Constraints are in reverse-source order. Due to TypeGuard semantics
-        // constraint AND is non-commutative and so we _must_ apply in
-        // source order.
-        //
-        // Fortunately, constraint AND is still associative, so we can still iterate left-to-right
-        // and accumulate rightward.
-        self.filter_map(|constraint| infer_narrowing_constraint(db, constraint, place))
-            .reduce(|acc, constraint| {
-                // See above---note the reverse application
-                constraint.merge_constraint_and(acc, db)
-            })
-            .map_or(base_ty, |constraint| {
-                NarrowingConstraint::regular(base_ty)
-                    .merge_constraint_and(constraint, db)
-                    .evaluate_constraint_type(db)
-            })
+        self.reachability_constraints.narrow_by_constraint(
+            db,
+            self.predicates,
+            self.constraint,
+            base_ty,
+            place,
+        )
     }
 }
 
@@ -840,9 +813,6 @@ pub(super) struct UseDefMapBuilder<'db> {
     /// Builder of predicates.
     pub(super) predicates: PredicatesBuilder<'db>,
 
-    /// Builder of narrowing constraints.
-    pub(super) narrowing_constraints: NarrowingConstraintsBuilder,
-
     /// Builder of reachability constraints.
     pub(super) reachability_constraints: ReachabilityConstraintsBuilder,
 
@@ -885,7 +855,6 @@ impl<'db> UseDefMapBuilder<'db> {
         Self {
             all_definitions: IndexVec::from_iter([DefinitionState::Undefined]),
             predicates: PredicatesBuilder::default(),
-            narrowing_constraints: NarrowingConstraintsBuilder::default(),
             reachability_constraints: ReachabilityConstraintsBuilder::default(),
             bindings_by_use: IndexVec::new(),
             reachability: ScopedReachabilityConstraintId::ALWAYS_TRUE,
@@ -1002,7 +971,12 @@ impl<'db> UseDefMapBuilder<'db> {
         }
     }
 
-    pub(super) fn record_narrowing_constraint(&mut self, predicate: ScopedPredicateId) {
+    /// Records a narrowing constraint for only the specified places.
+    pub(super) fn record_narrowing_constraint_for_places(
+        &mut self,
+        predicate: ScopedPredicateId,
+        places: &PossiblyNarrowedPlaces,
+    ) {
         if predicate == ScopedPredicateId::ALWAYS_TRUE
             || predicate == ScopedPredicateId::ALWAYS_FALSE
         {
@@ -1010,15 +984,57 @@ impl<'db> UseDefMapBuilder<'db> {
             return;
         }
 
-        let narrowing_constraint = predicate.into();
-        for state in &mut self.symbol_states {
-            state
-                .record_narrowing_constraint(&mut self.narrowing_constraints, narrowing_constraint);
+        let atom = self.reachability_constraints.add_atom(predicate);
+        self.record_narrowing_constraint_node_for_places(atom, places);
+    }
+
+    /// Records a negated narrowing constraint for only the specified places.
+    ///
+    /// Uses TDD-level negation (`add_not_constraint`) rather than creating a new predicate atom
+    /// for the negated predicate. This ensures that `atom(P) OR NOT(atom(P))` simplifies to
+    /// `ALWAYS_TRUE` in the TDD, so narrowing is correctly cancelled out after complete
+    /// if/else blocks.
+    pub(super) fn record_negated_narrowing_constraint_for_places(
+        &mut self,
+        predicate: ScopedPredicateId,
+        places: &PossiblyNarrowedPlaces,
+    ) {
+        if predicate == ScopedPredicateId::ALWAYS_TRUE
+            || predicate == ScopedPredicateId::ALWAYS_FALSE
+        {
+            return;
         }
 
-        for state in &mut self.member_states {
-            state
-                .record_narrowing_constraint(&mut self.narrowing_constraints, narrowing_constraint);
+        let atom = self.reachability_constraints.add_atom(predicate);
+        let negated = self.reachability_constraints.add_not_constraint(atom);
+        self.record_narrowing_constraint_node_for_places(negated, places);
+    }
+
+    /// Records a TDD narrowing constraint node for the specified places.
+    fn record_narrowing_constraint_node_for_places(
+        &mut self,
+        constraint: ScopedNarrowingConstraint,
+        places: &PossiblyNarrowedPlaces,
+    ) {
+        for place in places {
+            match place {
+                ScopedPlaceId::Symbol(symbol_id) => {
+                    if let Some(state) = self.symbol_states.get_mut(*symbol_id) {
+                        state.record_narrowing_constraint(
+                            &mut self.reachability_constraints,
+                            constraint,
+                        );
+                    }
+                }
+                ScopedPlaceId::Member(member_id) => {
+                    if let Some(state) = self.member_states.get_mut(*member_id) {
+                        state.record_narrowing_constraint(
+                            &mut self.reachability_constraints,
+                            constraint,
+                        );
+                    }
+                }
+            }
         }
     }
 
@@ -1094,11 +1110,7 @@ impl<'db> UseDefMapBuilder<'db> {
             negated_reachability_id,
         );
 
-        self.symbol_states[symbol].merge(
-            post_definition_state,
-            &mut self.narrowing_constraints,
-            &mut self.reachability_constraints,
-        );
+        self.symbol_states[symbol].merge(post_definition_state, &mut self.reachability_constraints);
 
         // And similarly for all associated members:
         for (member_id, pre_definition_member_state) in pre_definition.associated_member_states {
@@ -1117,11 +1129,26 @@ impl<'db> UseDefMapBuilder<'db> {
                 negated_reachability_id,
             );
 
-            self.member_states[member_id].merge(
-                post_definition_state,
-                &mut self.narrowing_constraints,
-                &mut self.reachability_constraints,
-            );
+            self.member_states[member_id]
+                .merge(post_definition_state, &mut self.reachability_constraints);
+        }
+    }
+
+    /// Records a narrowing constraint for all places in the current scope.
+    ///
+    /// This is used to gate narrowing by `ReturnsNever` constraints: when a branch contains
+    /// a call to a `NoReturn` function, all narrowing in that branch should be conditional
+    /// on the call actually returning `Never`.
+    pub(super) fn record_narrowing_constraint_for_all_places(
+        &mut self,
+        constraint: ScopedNarrowingConstraint,
+    ) {
+        for state in &mut self.symbol_states {
+            state.record_narrowing_constraint(&mut self.reachability_constraints, constraint);
+        }
+
+        for state in &mut self.member_states {
+            state.record_narrowing_constraint(&mut self.reachability_constraints, constraint);
         }
     }
 
@@ -1289,12 +1316,11 @@ impl<'db> UseDefMapBuilder<'db> {
                 let new_symbol_state = &self.symbol_states[enclosing_symbol];
                 bindings.merge(
                     new_symbol_state.bindings().clone(),
-                    &mut self.narrowing_constraints,
                     &mut self.reachability_constraints,
                 );
             }
             Some(EnclosingSnapshot::Constraint(constraint)) => {
-                *constraint = ScopedNarrowingConstraint::empty();
+                *constraint = ScopedNarrowingConstraint::ALWAYS_TRUE;
             }
             None => {}
         }
@@ -1361,15 +1387,10 @@ impl<'db> UseDefMapBuilder<'db> {
         let mut snapshot_definitions_iter = snapshot.symbol_states.into_iter();
         for current in &mut self.symbol_states {
             if let Some(snapshot) = snapshot_definitions_iter.next() {
-                current.merge(
-                    snapshot,
-                    &mut self.narrowing_constraints,
-                    &mut self.reachability_constraints,
-                );
+                current.merge(snapshot, &mut self.reachability_constraints);
             } else {
                 current.merge(
                     PlaceState::undefined(snapshot.reachability),
-                    &mut self.narrowing_constraints,
                     &mut self.reachability_constraints,
                 );
                 // Place not present in snapshot, so it's unbound/undeclared from that path.
@@ -1379,15 +1400,10 @@ impl<'db> UseDefMapBuilder<'db> {
         let mut snapshot_definitions_iter = snapshot.member_states.into_iter();
         for current in &mut self.member_states {
             if let Some(snapshot) = snapshot_definitions_iter.next() {
-                current.merge(
-                    snapshot,
-                    &mut self.narrowing_constraints,
-                    &mut self.reachability_constraints,
-                );
+                current.merge(snapshot, &mut self.reachability_constraints);
             } else {
                 current.merge(
                     PlaceState::undefined(snapshot.reachability),
-                    &mut self.narrowing_constraints,
                     &mut self.reachability_constraints,
                 );
                 // Place not present in snapshot, so it's unbound/undeclared from that path.
@@ -1459,7 +1475,6 @@ impl<'db> UseDefMapBuilder<'db> {
         UseDefMap {
             all_definitions: self.all_definitions,
             predicates: self.predicates.build(),
-            narrowing_constraints: self.narrowing_constraints.build(),
             reachability_constraints: self.reachability_constraints.build(),
             bindings_by_use: self.bindings_by_use,
             node_reachability: self.node_reachability,
