@@ -5,13 +5,14 @@ use crate::place::builtins_module_scope;
 use crate::semantic_index::definition::Definition;
 use crate::semantic_index::definition::DefinitionKind;
 use crate::semantic_index::{attribute_scopes, global_scope, semantic_index, use_def_map};
-use crate::types::call::{CallArguments, MatchedArgument};
+use crate::types::call::{CallArguments, CallError, MatchedArgument};
 use crate::types::signatures::{ParameterKind, Signature};
 use crate::types::{
     CallDunderError, CallableTypes, ClassBase, ClassLiteral, ClassType, KnownUnion, Type,
     TypeContext, UnionType,
 };
-use crate::{Db, DisplaySettings, HasType, SemanticModel};
+use crate::{Db, DisplaySettings, HasDefinition, HasType, SemanticModel};
+use itertools::Either;
 use ruff_db::files::FileRange;
 use ruff_db::parsed::parsed_module;
 use ruff_python_ast::{self as ast, AnyNodeRef};
@@ -229,18 +230,18 @@ pub fn definitions_for_attribute<'db>(
     };
 
     let tys = match lhs_ty {
-        Type::Union(union) => union.elements(model.db()).to_vec(),
-        _ => vec![lhs_ty],
+        Type::Union(union) => union.elements(model.db()),
+        _ => std::slice::from_ref(&lhs_ty),
     };
 
     // Expand intersections for each subtype into their components
     let expanded_tys = tys
-        .into_iter()
+        .iter()
         .flat_map(|ty| match ty {
-            Type::Intersection(intersection) => intersection.positive(db).iter().copied().collect(),
-            _ => vec![ty],
+            Type::Intersection(intersection) => Either::Left(intersection.positive(db).iter()),
+            _ => Either::Right(std::iter::once(ty)),
         })
-        .collect::<Vec<_>>();
+        .copied();
 
     for ty in expanded_tys {
         // Handle modules
@@ -259,12 +260,15 @@ pub fn definitions_for_attribute<'db>(
             continue;
         }
 
-        // First, transform the type to its meta type, unless it's already a class-like type.
-        let meta_type = match ty {
+        let meta_type = ty.to_meta_type(db);
+
+        // Look up the attribute first on the meta-type, unless it's already a class-like type.
+        let lookup_type = match ty {
             Type::ClassLiteral(_) | Type::SubclassOf(_) | Type::GenericAlias(_) => ty,
-            _ => ty.to_meta_type(db),
+            _ => meta_type,
         };
-        let class_literal = match meta_type {
+
+        let class_literal = match lookup_type {
             Type::ClassLiteral(class_literal) => class_literal,
             Type::SubclassOf(subclass) => match subclass.subclass_of().into_class(db) {
                 Some(cls) => match cls.static_class_literal(db) {
@@ -276,26 +280,105 @@ pub fn definitions_for_attribute<'db>(
             _ => continue,
         };
 
-        // Walk the MRO: include class and its ancestors, but stop when we find a match
-        'scopes: for ancestor in class_literal
-            .iter_mro(db)
-            .filter_map(ClassBase::into_class)
-            .filter_map(|cls: ClassType<'db>| cls.static_class_literal(db).map(|(lit, _)| lit))
-        {
-            let class_scope = ancestor.body_scope(db);
-            let class_place_table = crate::semantic_index::place_table(db, class_scope);
+        resolved.extend(definitions_for_attribute_in_class_hierarchy(
+            &class_literal,
+            model,
+            name_str,
+        ));
 
-            // Look for class-level declarations and bindings
-            if let Some(place_id) = class_place_table.symbol_id(name_str) {
-                let use_def = use_def_map(db, class_scope);
+        // The metaclass of a derived class must be a subclass of the metaclasses of all of
+        // its base classes. This is why we only have to look at the metaclass of the
+        // class_literal.
+        // Only look up definitions on the metaclass if the type is a class object to begin with in
+        // order to prevent looking up instance members on the class metaclass
+        if resolved.is_empty() && meta_type != lookup_type {
+            let class_literal = match meta_type {
+                Type::ClassLiteral(class_literal) => class_literal,
+                Type::SubclassOf(subclass) => match subclass.subclass_of().into_class(db) {
+                    Some(cls) => match cls.static_class_literal(db) {
+                        Some((lit, _)) => ClassLiteral::Static(lit),
+                        None => continue,
+                    },
+                    None => continue,
+                },
+                _ => continue,
+            };
+
+            resolved.extend(definitions_for_attribute_in_class_hierarchy(
+                &class_literal,
+                model,
+                name_str,
+            ));
+        }
+    }
+
+    resolved
+}
+
+fn definitions_for_attribute_in_class_hierarchy<'db>(
+    class_literal: &ClassLiteral<'db>,
+    model: &SemanticModel<'db>,
+    attribute_name: &str,
+) -> Vec<ResolvedDefinition<'db>> {
+    let db = model.db();
+    let mut resolved = Vec::new();
+    'scopes: for ancestor in class_literal
+        .iter_mro(db)
+        .filter_map(ClassBase::into_class)
+        .filter_map(|cls: ClassType<'db>| cls.static_class_literal(db).map(|(lit, _)| lit))
+    {
+        let class_scope = ancestor.body_scope(db);
+        let class_place_table = crate::semantic_index::place_table(db, class_scope);
+
+        // Look for class-level declarations and bindings
+        if let Some(place_id) = class_place_table.symbol_id(attribute_name) {
+            let use_def = use_def_map(db, class_scope);
+
+            // Check declarations first
+            for decl in use_def.reachable_symbol_declarations(place_id) {
+                if let Some(def) = decl.declaration.definition() {
+                    resolved.extend(resolve_definition(
+                        db,
+                        def,
+                        Some(attribute_name),
+                        ImportAliasResolution::ResolveAliases,
+                    ));
+                    break 'scopes;
+                }
+            }
+
+            // If no declarations found, check bindings
+            for binding in use_def.reachable_symbol_bindings(place_id) {
+                if let Some(def) = binding.binding.definition() {
+                    resolved.extend(resolve_definition(
+                        db,
+                        def,
+                        Some(attribute_name),
+                        ImportAliasResolution::ResolveAliases,
+                    ));
+                    break 'scopes;
+                }
+            }
+        }
+
+        // Look for instance attributes in method scopes (e.g., self.x = 1)
+        let file = class_scope.file(db);
+        let index = semantic_index(db, file);
+
+        for function_scope_id in attribute_scopes(db, class_scope) {
+            if let Some(place_id) = index
+                .place_table(function_scope_id)
+                .member_id_by_instance_attribute_name(attribute_name)
+            {
+                let use_def = index.use_def_map(function_scope_id);
 
                 // Check declarations first
-                for decl in use_def.reachable_symbol_declarations(place_id) {
+                for decl in use_def.reachable_member_declarations(place_id) {
                     if let Some(def) = decl.declaration.definition() {
                         resolved.extend(resolve_definition(
                             db,
                             def,
-                            Some(name_str),
+                            Some(attribute_name),
                             ImportAliasResolution::ResolveAliases,
                         ));
                         break 'scopes;
@@ -303,65 +386,67 @@ pub fn definitions_for_attribute<'db>(
                 }
 
                 // If no declarations found, check bindings
-                for binding in use_def.reachable_symbol_bindings(place_id) {
+                for binding in use_def.reachable_member_bindings(place_id) {
                     if let Some(def) = binding.binding.definition() {
                         resolved.extend(resolve_definition(
                             db,
                             def,
-                            Some(name_str),
+                            Some(attribute_name),
                             ImportAliasResolution::ResolveAliases,
                         ));
                         break 'scopes;
                     }
                 }
             }
-
-            // Look for instance attributes in method scopes (e.g., self.x = 1)
-            let file = class_scope.file(db);
-            let index = semantic_index(db, file);
-
-            for function_scope_id in attribute_scopes(db, class_scope) {
-                if let Some(place_id) = index
-                    .place_table(function_scope_id)
-                    .member_id_by_instance_attribute_name(name_str)
-                {
-                    let use_def = index.use_def_map(function_scope_id);
-
-                    // Check declarations first
-                    for decl in use_def.reachable_member_declarations(place_id) {
-                        if let Some(def) = decl.declaration.definition() {
-                            resolved.extend(resolve_definition(
-                                db,
-                                def,
-                                Some(name_str),
-                                ImportAliasResolution::ResolveAliases,
-                            ));
-                            break 'scopes;
-                        }
-                    }
-
-                    // If no declarations found, check bindings
-                    for binding in use_def.reachable_member_bindings(place_id) {
-                        if let Some(def) = binding.binding.definition() {
-                            resolved.extend(resolve_definition(
-                                db,
-                                def,
-                                Some(name_str),
-                                ImportAliasResolution::ResolveAliases,
-                            ));
-                            break 'scopes;
-                        }
-                    }
-                }
-            }
-
-            // TODO: Add support for metaclass attribute lookups
         }
     }
 
     resolved
 }
 
+pub struct TypedDictKeyHover<'db> {
+    pub owner: String,
+    pub key: String,
+    pub declared_ty: Type<'db>,
+    pub docstring: Option<String>,
+}
+
+pub fn typed_dict_key_definition<'db>(
+    model: &SemanticModel<'db>,
+    subscript: &ast::ExprSubscript,
+    key: &str,
+) -> Option<ResolvedDefinition<'db>> {
+    let value_ty = subscript.value.inferred_type(model)?;
+    let typed_dict = value_ty.as_typed_dict()?;
+    let field = typed_dict.items(model.db()).get(key)?;
+    let definition = field.first_declaration()?;
+    Some(ResolvedDefinition::Definition(definition))
+}
+
+pub fn typed_dict_key_hover<'db>(
+    model: &SemanticModel<'db>,
+    subscript: &ast::ExprSubscript,
+) -> Option<TypedDictKeyHover<'db>> {
+    let key = subscript
+        .slice
+        .as_string_literal_expr()
+        .map(|literal| literal.value.to_str())?;
+    let value_ty = subscript.value.inferred_type(model)?;
+    let typed_dict = value_ty.as_typed_dict()?;
+    let owner = value_ty.display(model.db()).to_string();
+    let field = typed_dict.items(model.db()).get(key)?;
+    let docstring = field
+        .first_declaration()
+        .and_then(|declaration| declaration.docstring(model.db()));
+
+    Some(TypedDictKeyHover {
+        owner,
+        key: key.to_string(),
+        declared_ty: field.declared_ty,
+        docstring,
+    })
+}
+
 /// Returns definitions for a keyword argument in a call expression.
 /// This resolves the keyword argument to the corresponding parameter(s) in the callable's signature(s).
 pub fn definitions_for_keyword_argument<'db>(
@@ -439,6 +524,28 @@ pub fn definitions_for_imported_symbol<'db>(
     )
 }
 
+/// Returns the definition and overload co-definitions for a function declaration.
+///
+/// For overloaded functions this includes sibling overload declarations and the
+/// implementation, if present.
+pub fn definitions_and_overloads_for_function<'db>(
+    model: &SemanticModel<'db>,
+    function: &ast::StmtFunctionDef,
+) -> Vec<ResolvedDefinition<'db>> {
+    if let Some(function_type) = function
+        .inferred_type(model)
+        .and_then(Type::as_function_literal)
+    {
+        function_type
+            .iter_overloads_and_implementation(model.db())
+            .filter_map(|overload| overload.signature(model.db()).definition())
+            .map(ResolvedDefinition::Definition)
+            .collect()
+    } else {
+        vec![ResolvedDefinition::Definition(function.definition(model))]
+    }
+}
+
 /// Details about a callable signature for IDE support.
 #[derive(Debug, Clone)]
 pub struct CallSignatureDetails<'db> {
@@ -472,7 +579,29 @@ pub struct CallSignatureDetails<'db> {
     pub argument_to_parameter_mapping: Vec<MatchedArgument<'db>>,
 }
 
-impl CallSignatureDetails<'_> {
+impl<'db> CallSignatureDetails<'db> {
+    fn from_binding(db: &'db dyn Db, binding: &crate::types::call::Binding<'db>) -> Self {
+        let argument_to_parameter_mapping = binding.argument_matches().to_vec();
+        let signature = binding.signature.clone();
+        let display_details = signature.display(db).to_string_parts();
+        let (parameter_kinds, parameter_types): (Vec<ParameterKind>, Vec<Type>) = signature
+            .parameters()
+            .iter()
+            .map(|param| (param.kind().clone(), param.annotated_type()))
+            .unzip();
+
+        CallSignatureDetails {
+            definition: signature.definition(),
+            signature,
+            label: display_details.label,
+            parameter_label_offsets: display_details.parameter_ranges,
+            parameter_names: display_details.parameter_names,
+            parameter_kinds,
+            parameter_types,
+            argument_to_parameter_mapping,
+        }
+    }
+
     fn get_definition_parameter_range(&self, db: &dyn Db, name: &str) -> Option<FileRange> {
         let definition = self.signature.definition()?;
         let file = definition.file(db);
@@ -500,10 +629,12 @@ pub fn call_signature_details<'db>(
         return Vec::new();
     };
 
+    let db = model.db();
+
     // Use into_callable to handle all the complex type conversions
     if let Some(callable_type) = func_type
-        .try_upcast_to_callable(model.db())
-        .map(|callables| callables.into_type(model.db()))
+        .try_upcast_to_callable(db)
+        .map(|callables| callables.into_type(db))
     {
         let call_arguments =
             CallArguments::from_arguments(&call_expr.arguments, |_, splatted_value| {
@@ -512,36 +643,14 @@ pub fn call_signature_details<'db>(
                     .unwrap_or(Type::unknown())
             });
         let bindings = callable_type
-            .bindings(model.db())
-            .match_parameters(model.db(), &call_arguments);
+            .bindings(db)
+            .match_parameters(db, &call_arguments);
 
         // Extract signature details from all callable bindings
         bindings
             .into_iter()
             .flatten()
-            .map(|binding| {
-                let argument_to_parameter_mapping = binding.argument_matches().to_vec();
-                let signature = binding.signature;
-                let display_details = signature.display(model.db()).to_string_parts();
-                let parameter_label_offsets = display_details.parameter_ranges;
-                let parameter_names = display_details.parameter_names;
-                let (parameter_kinds, parameter_types): (Vec<ParameterKind>, Vec<Type>) = signature
-                    .parameters()
-                    .iter()
-                    .map(|param| (param.kind().clone(), param.annotated_type()))
-                    .unzip();
-
-                CallSignatureDetails {
-                    definition: signature.definition(),
-                    signature,
-                    label: display_details.label,
-                    parameter_label_offsets,
-                    parameter_names,
-                    parameter_kinds,
-                    parameter_types,
-                    argument_to_parameter_mapping,
-                }
-            })
+            .map(|binding| CallSignatureDetails::from_binding(db, &binding))
             .collect()
     } else {
         // Type is not callable, return empty signatures
@@ -579,7 +688,7 @@ pub fn call_type_simplified_by_overloads(
     }
 
     // Hand the overload resolution system as much type info as we have
-    let args = CallArguments::from_arguments_typed(&call_expr.arguments, |_, splatted_value| {
+    let args = CallArguments::from_arguments_typed(&call_expr.arguments, |splatted_value| {
         splatted_value
             .inferred_type(model)
             .unwrap_or(Type::unknown())
@@ -754,6 +863,60 @@ pub fn find_active_signature_from_details(
     Some(best_index)
 }
 
+/// Resolve a call expression to its matching overload's signature details,
+/// using full type checking (not just arity matching) for overload resolution.
+///
+/// Falls back to arity-based matching if type-based resolution fails.
+fn resolve_call_signature<'db>(
+    model: &SemanticModel<'db>,
+    call_expr: &ast::ExprCall,
+) -> Option<CallSignatureDetails<'db>> {
+    let db = model.db();
+    let func_type = call_expr.func.inferred_type(model)?;
+    let callable_type = func_type.try_upcast_to_callable(db)?.into_type(db);
+
+    let args = CallArguments::from_arguments_typed(&call_expr.arguments, |splatted_value| {
+        splatted_value
+            .inferred_type(model)
+            .unwrap_or(Type::unknown())
+    });
+
+    // Extract the `Bindings` regardless of whether type checking succeeded or failed.
+    let bindings = callable_type
+        .bindings(db)
+        .match_parameters(db, &args)
+        .check_types(db, &args, TypeContext::default(), &[])
+        .unwrap_or_else(|CallError(_, bindings)| *bindings);
+
+    // First, try to find the matching overload after full type checking.
+    let type_checked_details: Vec<_> = bindings
+        .iter()
+        .flat_map(|binding| binding.matching_overloads().map(|(_, overload)| overload))
+        .map(|binding| CallSignatureDetails::from_binding(db, binding))
+        .collect();
+
+    if !type_checked_details.is_empty() {
+        let active = find_active_signature_from_details(&type_checked_details)?;
+        return type_checked_details.into_iter().nth(active);
+    }
+
+    // If all overloads have type-checking errors (e.g., `InvalidArgumentType`),
+    // `matching_overloads()` returns empty. Fall back to arity-based matching
+    // across all overloads to pick the best candidate for showing hints.
+    let all_details: Vec<_> = bindings
+        .iter()
+        .flatten()
+        .map(|binding| CallSignatureDetails::from_binding(db, binding))
+        .collect();
+
+    if all_details.is_empty() {
+        return None;
+    }
+
+    let active = find_active_signature_from_details(&all_details)?;
+    all_details.into_iter().nth(active)
+}
+
 #[derive(Default)]
 pub struct InlayHintCallArgumentDetails {
     /// The position of the arguments mapped to their name and the range of the argument definition in the signature.
@@ -765,25 +928,14 @@ pub fn inlay_hint_call_argument_details<'db>(
     model: &SemanticModel<'db>,
     call_expr: &ast::ExprCall,
 ) -> Option<InlayHintCallArgumentDetails> {
-    let signature_details = call_signature_details(model, call_expr);
+    let resolved = resolve_call_signature(model, call_expr)?;
 
-    if signature_details.is_empty() {
-        return None;
-    }
-
-    let active_signature_index = find_active_signature_from_details(&signature_details)?;
-
-    let call_signature_details = signature_details.get(active_signature_index)?;
-
-    let parameters = call_signature_details.signature.parameters();
+    let parameters = resolved.signature.parameters();
 
     let mut argument_names = HashMap::new();
 
     for arg_index in 0..call_expr.arguments.args.len() {
-        let Some(arg_mapping) = call_signature_details
-            .argument_to_parameter_mapping
-            .get(arg_index)
-        else {
+        let Some(arg_mapping) = resolved.argument_to_parameter_mapping.get(arg_index) else {
             continue;
         };
 
@@ -805,8 +957,7 @@ pub fn inlay_hint_call_argument_details<'db>(
             continue;
         };
 
-        let parameter_label_offset =
-            call_signature_details.get_definition_parameter_range(db, param.name()?);
+        let parameter_label_offset = resolved.get_definition_parameter_range(db, param.name()?);
 
         // Only add hints for parameters that can be specified by name
         if !param.is_positional_only() && !param.is_variadic() && !param.is_keyword_variadic() {
@@ -878,7 +1029,7 @@ mod resolve_definition {
     }
 
     impl<'db> ResolvedDefinition<'db> {
-        pub(crate) fn definition(&self) -> Option<Definition<'db>> {
+        pub fn definition(&self) -> Option<Definition<'db>> {
             match self {
                 ResolvedDefinition::Definition(definition) => Some(*definition),
                 ResolvedDefinition::Module(_) => None,
@@ -1367,6 +1518,7 @@ mod resolve_definition {
             | DefinitionKind::Assignment(_)
             | DefinitionKind::AnnotatedAssignment(_)
             | DefinitionKind::AugmentedAssignment(_)
+            | DefinitionKind::DictKeyAssignment(_)
             | DefinitionKind::For(_)
             | DefinitionKind::Comprehension(_)
             | DefinitionKind::VariadicPositionalParameter(_)
