@@ -235,6 +235,7 @@ impl<'db> ProtocolInterface<'db> {
                     ProtocolMemberData {
                         qualifiers: TypeQualifiers::default(),
                         kind: ProtocolMemberKind::Property(property),
+                        definition: None,
                     },
                 )
             })
@@ -257,14 +258,22 @@ impl<'db> ProtocolInterface<'db> {
             name,
             kind: data.kind,
             qualifiers: data.qualifiers,
+            definition: data.definition,
         })
     }
 
+    pub(super) fn non_method_members(self, db: &'db dyn Db) -> Vec<ProtocolMember<'db, 'db>> {
+        self.members(db)
+            .filter(|member| !member.is_method() && !member.ty().is_todo())
+            .collect()
+    }
+
     fn member_by_name<'a>(self, db: &'db dyn Db, name: &'a str) -> Option<ProtocolMember<'a, 'db>> {
         self.inner(db).get(name).map(|data| ProtocolMember {
             name,
             kind: data.kind,
             qualifiers: data.qualifiers,
+            definition: data.definition,
         })
     }
 
@@ -272,6 +281,15 @@ impl<'db> ProtocolInterface<'db> {
         self.inner(db).contains_key(name)
     }
 
+    /// Returns the `__call__` method's callable type if this protocol has a `__call__` method member.
+    pub(super) fn call_method(self, db: &'db dyn Db) -> Option<CallableType<'db>> {
+        self.member_by_name(db, "__call__")
+            .and_then(|member| match member.kind {
+                ProtocolMemberKind::Method(callable) => Some(callable),
+                _ => None,
+            })
+    }
+
     pub(super) fn instance_member(self, db: &'db dyn Db, name: &str) -> PlaceAndQualifiers<'db> {
         self.member_by_name(db, name)
             .map(|member| PlaceAndQualifiers {
@@ -488,6 +506,7 @@ impl<'db> VarianceInferable<'db> for ProtocolInterface<'db> {
 pub(super) struct ProtocolMemberData<'db> {
     kind: ProtocolMemberKind<'db>,
     qualifiers: TypeQualifiers,
+    definition: Option<Definition<'db>>,
 }
 
 impl<'db> ProtocolMemberData<'db> {
@@ -499,6 +518,7 @@ impl<'db> ProtocolMemberData<'db> {
         Self {
             kind: self.kind.normalized_impl(db, visitor),
             qualifiers: self.qualifiers,
+            definition: None,
         }
     }
 
@@ -525,6 +545,7 @@ impl<'db> ProtocolMemberData<'db> {
                 ),
             },
             qualifiers: self.qualifiers,
+            definition: self.definition,
         })
     }
 
@@ -540,6 +561,7 @@ impl<'db> ProtocolMemberData<'db> {
                 .kind
                 .apply_type_mapping_impl(db, type_mapping, tcx, visitor),
             qualifiers: self.qualifiers,
+            definition: self.definition,
         }
     }
 
@@ -669,6 +691,7 @@ pub(super) struct ProtocolMember<'a, 'db> {
     name: &'a str,
     kind: ProtocolMemberKind<'db>,
     qualifiers: TypeQualifiers,
+    definition: Option<Definition<'db>>,
 }
 
 fn walk_protocol_member<'db, V: super::visitor::TypeVisitor<'db> + ?Sized>(
@@ -694,6 +717,14 @@ impl<'a, 'db> ProtocolMember<'a, 'db> {
         self.qualifiers
     }
 
+    pub(super) const fn is_method(&self) -> bool {
+        matches!(self.kind, ProtocolMemberKind::Method(_))
+    }
+
+    pub(super) fn definition(&self) -> Option<Definition<'db>> {
+        self.definition
+    }
+
     fn ty(&self) -> Type<'db> {
         match &self.kind {
             ProtocolMemberKind::Method(callable) => Type::Callable(*callable),
@@ -919,32 +950,42 @@ fn cached_protocol_interface<'db>(
         // type narrowing that uses `isinstance()` or `issubclass()` with
         // runtime-checkable protocols.
         for (symbol_id, bindings) in use_def_map.all_end_of_scope_symbol_bindings() {
-            let Some(ty) = place_from_bindings(db, bindings)
-                .place
-                .ignore_possibly_undefined()
-            else {
+            let place_and_definition = place_from_bindings(db, bindings);
+            let Some(ty) = place_and_definition.place.ignore_possibly_undefined() else {
                 continue;
             };
             direct_members.insert(
                 symbol_id,
-                (ty, TypeQualifiers::default(), BoundOnClass::Yes),
+                (
+                    ty,
+                    TypeQualifiers::default(),
+                    place_and_definition.first_definition,
+                    BoundOnClass::Yes,
+                ),
             );
         }
 
         for (symbol_id, declarations) in use_def_map.all_end_of_scope_symbol_declarations() {
-            let place = place_from_declarations(db, declarations).ignore_conflicting_declarations();
+            let place_result = place_from_declarations(db, declarations);
+            let first_declaration = place_result.first_declaration;
+            let place = place_result.ignore_conflicting_declarations();
             if let Some(new_type) = place.place.ignore_possibly_undefined() {
                 direct_members
                     .entry(symbol_id)
-                    .and_modify(|(ty, quals, _)| {
+                    .and_modify(|(ty, quals, _, _)| {
                         *ty = new_type;
                         *quals = place.qualifiers;
                     })
-                    .or_insert((new_type, place.qualifiers, BoundOnClass::No));
+                    .or_insert((
+                        new_type,
+                        place.qualifiers,
+                        first_declaration,
+                        BoundOnClass::No,
+                    ));
             }
         }
 
-        for (symbol_id, (ty, qualifiers, bound_on_class)) in direct_members {
+        for (symbol_id, (ty, qualifiers, definition, bound_on_class)) in direct_members {
             let name = place_table.symbol(symbol_id).name();
             if excluded_from_proto_members(name) {
                 continue;
@@ -980,6 +1021,7 @@ fn cached_protocol_interface<'db>(
                 ProtocolMemberData {
                     kind: member,
                     qualifiers,
+                    definition,
                 },
             );
         }
